/*
AIR BRAKE
CTR-style air-braking
*/
local cv_airbrake = CV_RegisterVar({
    name = "rr_airbrake",
    defaultvalue = "On",
    possiblevalue = CV_OnOff,
    flags = CV_NETVAR
})

freeslot("MT_AIRBRAKEDUSTTRAIL", "SPR_ARBK", "S_AIRBRAKEDUST1", "S_AIRBRAKEDUST2", "S_FADEDUST1", "S_FADEDUST2");
states[S_AIRBRAKEDUST1] = {SPR_ARBK, B|FF_FULLBRIGHT, 1, nil, 0, 0, S_AIRBRAKEDUST2}
states[S_AIRBRAKEDUST2] = {SPR_ARBK, A|FF_FULLBRIGHT|FF_ANIMATE, 4, nil, 8, 1, S_NULL}

mobjinfo[MT_AIRBRAKEDUSTTRAIL] = {
		doomednum = -1,
		spawnstate = S_AIRBRAKEDUST1,    // spawnstate
		spawnhealth = 1,              // spawnhealth
		reactiontime = 8,              // reactiontime
		speed = 8,              // speed
		radius = 14*FRACUNIT,    // radius
		height = 14*FRACUNIT,    // height
		dispoffset = 0,              // display offset
		mass = 100,            // mass
		flags = MF_NOBLOCKMAP|MF_NOCLIP|MF_NOCLIPHEIGHT|MF_NOGRAVITY|MF_SCENERY|MF_SLIDEME|MF_DONTENCOREMAP, // flags
		raisestate = S_NULL          // raisestate
}


// lua'd this from the source
// Adds gravity flipping to an object relative to its master and shifts the z coordinate accordingly.
rawset(_G, "K_FlipFromObject", function(mo, master)
	mo.eflags = (mo.eflags & ~MFE_VERTICALFLIP)|(master.eflags & MFE_VERTICALFLIP)
	mo.flags2 = (mo.flags2 & ~MF2_OBJECTFLIP)|(master.flags2 & MF2_OBJECTFLIP)

	if (mo.eflags & MFE_VERTICALFLIP)
		mo.z = ($ + master.height - FixedMul(master.scale, mo.height))
	end
end)

rawset(_G, "K_SpawnAirBrakeDust", function(player, colorize) -- adjustable boost trail
	local newx --fixed_t
	local newy --fixed_t
	local ground --fixed_t
	local flame -- obj
	local travelangle;-- angle

	if (not (player.mo and player.mo.valid)) then return end;

	if ((G_BattleGametype() and player.kartstuff[k_bumper] <= 0 and player.kartstuff[k_comebacktimer])) // show sneaker boosts during a hyuu
		return;
    end

	if (player.mo.eflags & MFE_VERTICALFLIP)
		ground = player.mo.ceilingz - FixedMul(mobjinfo[MT_SNEAKERTRAIL].height, player.mo.scale);
	else
		ground = player.mo.floorz
    end

	if (player.kartstuff[k_drift] != 0)
		travelangle = player.mo.angle
	else
		travelangle = R_PointToAngle2(0, 0, player.rmomx, player.rmomy)
	end
	
	for j = 0, min(3, max(0, ((player.brakeholdtime) and (( ((fade) and 15 or player.brakeholdtime) ) / (TICRATE/4)) or 0) ))
		for i = 0, 2
			local tx = P_ReturnThrustX(player.mo, travelangle, FixedMul((-12*FRACUNIT)*(j), player.mo.scale))
			local ty = P_ReturnThrustY(player.mo, travelangle, FixedMul((-12*FRACUNIT)*(j), player.mo.scale));
			local tz = player.mo.momz * j;
			
			newx = player.mo.x + P_ReturnThrustX(player.mo, travelangle + ((i&1) and -1 or 1)*ANGLE_135, FixedMul((24*FRACUNIT), player.mo.scale)) + tx;
			newy = player.mo.y + P_ReturnThrustY(player.mo, travelangle + ((i&1) and -1 or 1)*ANGLE_135, FixedMul((24*FRACUNIT), player.mo.scale)) + ty;
			if (player.mo.standingslope)
				ground = P_GetZAt(player.mo.standingslope, newx, newy);
				if (player.mo.eflags & MFE_VERTICALFLIP)
					ground = ($ - FixedMul(mobjinfo[MT_SNEAKERTRAIL].height, player.mo.scale));
				end
			end
			flame = P_SpawnMobj(newx, newy, player.mo.z-(tz/4), MT_AIRBRAKEDUSTTRAIL);
			
			if (colorize)
				local setcolor = ColorOpposite(player.skincolor);
				flame.color = setcolor;
				flame.colorized = true;
			end

			flame.target = player.mo;
			flame.angle = travelangle;
			flame.sloperoll = player.mo.sloperoll;
			flame.fuse = TICRATE*2;
			flame.scale = player.mo.scale;
			P_SetScale(flame, player.mo.scale);
			// not K_MatchGenericExtraFlags so that a stolen sneaker can be seen
			K_FlipFromObject(flame, player.mo);

			flame.momx = 8;

			if (not (flame and flame.valid)) then continue end;
		end
	end
end)

local ANG1 = ANG1;
local ANGTenth = ANG1/10;
local ORIG_FRICTION = 62914;
local brakedelaytime = TICRATE/3;
local TICRATE_h = TICRATE/2;

local function midairBrake(p)
	if not cv_airbrake.value then return end
	if (not (p.mo and p.mo.valid)) then return; end
	if (p.grinding) or (p.railjumped) then return; end --Air brakes don't like rails much
	if (p.mo.paragliding) then return; end --Air brakes don't like gliders much
    if (p.graple and p.graple.valid) or (p.grapple and p.grapple.valid) then return end --Air brakes don't like grapple much
	if p.kartstuff[k_waterskip] ~= 0 then
        if not jug_nowaterskip or jug_nowaterskip.isWaterSkipEnabled then return end --Air brakes don't like water skip much
    end
    
    local pks = p.kartstuff;
	local doBrakeDelay = (p.brakedelay)
	local pflip;
	local cantairbrake = (pks[k_spinouttimer] or p.playerstate == PST_DEAD or pks[k_respawn])
	local noaccel = false;	
		
	if (not (p.cmd.buttons & BT_ACCELERATE))	
		noaccel = true;	
		//print("no accel");	
	end
	
	if (pks[k_spinouttimer]) then return; end
	
	pflip = (p.mo.eflags & MFE_VERTICALFLIP);
	
	if (not P_IsObjectOnGround(p.mo))
	and (p.cmd.buttons & BT_BRAKE)
		if (not p.brakedelayset)
			p.brakedelay = brakedelaytime;
			p.brakedelayset = true;
		end

		local dir = p.kartstuff[k_throwdir]*-1*P_MobjFlip(p.mo)
		
		if ((dir) and (not doBrakeDelay) and (not cantairbrake)) then
			p.brakeholdtime = ((p.brakeholdtime) and ($ + 1) or 1);
			K_SpawnAirBrakeDust(p, ((p.airbrakesneakertime) and true or false));	
			// you instead get heavier	
			local fastfall = dir == -1 and gravity*2 or gravity/2
			
			P_SetObjectMomZ(p.mo, dir*fastfall, true)
		end
	else
		p.brakedelayset = false;
		p.ab_speedmem = p.speed;
		p.airbrakesneakertime = nil;
		
		p.brakeholdtime = 0;
		p.ab_zdist = nil;
		p.ab_speedmem = p.speed;
	end
	
	if (p.brakedelay)
		p.brakedelay = $ - 1;
	end
end

local function airbrakepspawn(p)
	// clear out players when they spawn/respawn
	p.ab_zdist = nil;
	p.airbrakesneakertime = nil;
	p.pstackmem = nil;
	p.istackmem = nil;
	p.brakeholdtime = 0;
end

addHook("PlayerThink", midairBrake)
addHook("PlayerSpawn", airbrakepspawn)